#! /usr/bin/env python
#-*- encoding:utf-8 -*-

"""Stock Search

Description: 
"""
__version__  = "0.1"
__date__     = "2007-08-31"
__author__   = "Loya <loya.liu at gmail.com> "
__license__  = "Licensed under the GPL v2(http://www.opensource.org)."
__copyright__= "Copyright (C) 2007 by Loya."
#=================================================================================#


import sys
import urllib2


DEFAULT_CODE_LIST=['sh000001','sz399001']

#don't change this
hq_url = "http://hq.sinajs.cn/list=%s"

class Suggest:
	def __init__(self,path):
		self._load(path)
	def _load(self,path):
		fin=open(path)
		data=fin.read()
		data=data.decode('gbk')
		s=data.find('="')
		e=data.rfind('"')
		lstr=data[s+2:e]
		suggest_list=lstr.split('|')
		self.word_list=[]
		for x in suggest_list:
			if(len(x)==0):continue
			entry=x.split(':')
			key=entry[0]
			vl=entry[1].split('-')
			code=vl[0]+vl[1]
			self.word_list.append((key,code))
	def lookup(self,k):
		rlist=[]
		for x in self.word_list:
			key=x[0]
			code=x[1]
			if(key.find(k)>-1):rlist.append(code)
		return rlist
		
class StockData:
	def __init__(self,str_data,scode):
		self._parse(str_data)
		self.code=scode
		
	def _parse(self,str_data):
		s=str_data.find('="')
		s=s+2
		e=str_data.rfind('"')
		ss=str_data[s:e]
		data_list=ss.split(',')
		data_list[0]=data_list[0].decode('gbk')
		self._parse_data(data_list)
		
	def _parse_data(self,data_list):
		self.name=data_list[0]
		self.price_now=float(data_list[3])
		self.price_today_start=float(data_list[1])
		self.price_highest=float(data_list[4])
		self.price_lowest=float(data_list[5])
		self.price_lastday=float(data_list[2])
		self.change_percent=((self.price_now-self.price_lastday)/self.price_lastday)*100.0
		self.time=data_list[30]+" "+data_list[31]
		self.slot_buy=[]
		self.slot_sale=[]
		for i in range(0,5):
			self.slot_buy.append((float(data_list[11+i*2]),float(data_list[10+i*2])))
			self.slot_sale.append((float(data_list[21+i*2]),float(data_list[20+i*2])))

def getStockData(stockids):
	stockstr=','.join(stockids)
	url=hq_url % stockstr
	handle = urllib2.urlopen(url)
	buf = handle.read()
	listStr=buf.split(';')
	i=0
	rlist=[]
	while i<len(listStr)-1:
		str_data=listStr[i]
		try:
			sd=StockData(str_data,stockids[i])
			rlist.append(sd)
		except:
			pass #continue
		i=i+1
	return rlist

def print_st(stock_list):
	for x in stock_list:
		print "%s(%s)[%0.2f/%+0.2f%%/%0.2f/%0.2f/%0.2f]<%s>" % (x.code,x.name,x.price_now,x.change_percent,x.price_lastday,x.price_highest,x.price_lowest,x.time)
		#print x.slot_buy
		#print x.slot_sale

def process_suggest(code_list):
	su=Suggest('suggest.js')
	r=[]
	for x in code_list:
		l=su.lookup(x)
		if(len(l)>0):
			r=r+l
		else:
			print "> Error: Can't find stock code for %s." % x
	return r

def main():
	args=sys.argv[1:]
	code_list=[]
	if(len(args)>0):
		for x in args:
			al=x.split(',')
			for c in al:
				code_list.append(c)
	else:
		usage()
		code_list=DEFAULT_CODE_LIST
	if(len(code_list)>0):
		code_list=process_suggest(code_list)
		if(len(code_list)==0):
			print "> Warn:no stock to print."
			usage()
			return
		sl=getStockData(code_list)
		print_st(sl)

def usage():
	print 'usage: sts.py <[code,code,..]> '
	print '-----------------------------------------------------------------'
	print '     ex:  sts.py jxty'
	print '     ex:  sts.py sh600221,sh600222 sh600208'
	print '     ex:  sts.py jxty,sh600222 sh000001'
	print '-----------------------------------------------------------------'

if __name__=="__main__":
	main()
